// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "assert/assertgl.hpp"
#include "util/macro.hpp"

// Definition Helpers
#define SHADER_PARAMETER_BUFFER_ARRAY_DEFINE(name, size, contents) \
  struct MACRO_JOIN(name, _struct) { \
    contents \
  }; \
  struct MACRO_JOIN(name, _struct) name[size];

#define SHADER_PARAMETER_BUFFER_DEFINE(name, structure) \
  struct MACRO_JOIN(name, Data) {\
    structure; \
  }; \
  class name : public ShaderParameterBuffer<struct MACRO_JOIN(name, Data)> { };


// Integer and Integer Arrays
#ifdef DAWN_OPENGL_SHADER_BUFFER_INTEGER_PADDING
  #define SHADER_PARAMETER_BUFFER_INTEGER(name, i) \
    int32_t name; \
    int32_t MACRO_JOIN(padding, i)[DAWN_OPENGL_SHADER_BUFFER_INTEGER_PADDING];
#else
  #define SHADER_PARAMETER_BUFFER_INTEGER(name, i) int32_t name;
#endif

#define SHADER_PARAMETER_BUFFER_INTEGER_ARRAY(name, size) SHADER_PARAMETER_BUFFER_ARRAY_DEFINE(name, size, SHADER_PARAMETER_BUFFER_INTEGER(value, 0))

// Color and Color Arrays
#define SHADER_PARAMETER_BUFFER_COLOR(name) \
  struct Color name;

#define SHADER_PARAMETER_BUFFER_COLOR_ARRAY(name, size) \
  struct Color name[size];

// MAT4
#define SHADER_PARAMETER_BUFFER_MAT4(name) \
  glm::mat4 name;

// VEC2
#define SHADER_PARAMETER_BUFFER_VEC2(name) \
  glm::vec2 name;

#define SHADER_PARAMETER_BUFFER_VEC2_ARRAY(name, size) \
  glm::vec2 name[size];

// EOF Fix
#define NOTHING "Fixes an error with EOF"