// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "SimpleTexturedMaterial.hpp"
#include "game/DawnGame.hpp"

using namespace Dawn;

SimpleTexturedMaterial::SimpleTexturedMaterial(SceneItem *i) :
  Material(i)
{
}

void SimpleTexturedMaterial::onStart() {
  this->shaderLock = getShaderManager().lockShader<SimpleTexturedShader>();
}

void SimpleTexturedMaterial::onDispose() {
  getShaderManager().releaseShader<SimpleTexturedShader>(this->shaderLock);
}

std::vector<struct ShaderPassItem>
  SimpleTexturedMaterial::getRenderPasses(IRenderableContext &context)
{
  auto mesh = this->item->getComponent<MeshRenderer>();
  auto shader = getShaderManager()
    .getShader<SimpleTexturedShader>(this->shaderLock)
  ;

  assertNotNull(mesh, "Mesh cannot be null");
  assertNotNull(mesh->mesh, "Mesh cannot be null");
  assertNotNull(shader, "Shader cannot be null");

  struct ShaderPassItem onlyPass;
  onlyPass.mesh = mesh->mesh;
  onlyPass.shader = shader;
  onlyPass.colorValues[shader->paramColor] = this->color;
  onlyPass.matrixValues[shader->paramModel] =
    this->transform->getWorldTransform()
  ;
  onlyPass.parameterBuffers[
    shader->bufferRenderPipeline
  ] = &context.renderPipeline->shaderBuffer;

  if(this->opaque) {
    onlyPass.renderFlags = (
      RENDER_MANAGER_RENDER_FLAG_DEPTH_TEST
    );
  } else {
    onlyPass.priority = 100;
    onlyPass.renderFlags = (
      RENDER_MANAGER_RENDER_FLAG_BLEND
    );
  }


  if(this->texture != nullptr) {
    onlyPass.boolValues[shader->paramHasTexture] = true;
    onlyPass.textureSlots[0] = this->texture;
    onlyPass.textureValues[shader->paramTexture] = 0;
  } else {
    onlyPass.boolValues[shader->paramHasTexture] = false;
  }
  
  return { onlyPass };
}